#include <Wire.h>

// DS1388 アドレス
#define DS1388_ADDRESS          0x68

// DS1388 EEPROM レジスタ ビット
#define DS1388_EEPROM_0         0x01
#define DS1388_EEPROM_1         0x02

void setup() {
    Serial.begin(115200);
    Wire.begin();

    // RTC時刻調整
    adjustRtc(2017, 1, 1, 0, 0, 0);
}

void loop() {
    // RTC時刻取得
    if (!printRtc()) {
        Serial.println("printRtc error.");
    }
    delay(5000);
}

uint8_t bcd2bin(uint8_t val) {
    return val - 6 * (val >> 4);
}

uint8_t bin2bcd (uint8_t val) {
    return val + 6 * (val / 10);
}

// RTC時刻調整
boolean adjustRtc(uint16_t year, uint8_t month, uint8_t day, uint8_t hour, uint8_t min, uint8_t sec) {
    int res = 0;
    Wire.beginTransmission(DS1388_ADDRESS);
        Wire.write((byte) 0);
        Wire.write(bin2bcd(0));
        Wire.write(bin2bcd(sec));
        Wire.write(bin2bcd(min));
        Wire.write(bin2bcd(hour));
        Wire.write(bin2bcd(0));
        Wire.write(bin2bcd(day));
        Wire.write(bin2bcd(month));
        Wire.write(bin2bcd(year - 2000));
    res = Wire.endTransmission();
    if (res == 0) {
        Wire.beginTransmission(DS1388_ADDRESS);
        Wire.write((byte) 0x0b);
        Wire.write((byte) 0x00);
        res = Wire.endTransmission();
    }
    return res == 0;
}

// RTC時刻取得
boolean printRtc() {
    boolean result = false;
    int res = 0;
    Wire.beginTransmission(DS1388_ADDRESS);
        Wire.write((byte) 0);
    res = Wire.endTransmission();
    if (res == 0) {
        Wire.requestFrom(DS1388_ADDRESS, 8);
        if (Wire.available()) {
            uint8_t hs = bcd2bin(Wire.read() & 0x7F);
            uint8_t ss = bcd2bin(Wire.read() & 0x7F);
            uint8_t mm = bcd2bin(Wire.read());
            uint8_t hh = bcd2bin(Wire.read());
            Wire.read();
            uint8_t d = bcd2bin(Wire.read());
            uint8_t m = bcd2bin(Wire.read());
            uint16_t y = bcd2bin(Wire.read()) + 2000;  
            printDateTime(y, m, d, hh, mm, ss);
            result = true;
        }
    }
    return result;
}

void printDateTime(uint16_t year, uint8_t month, uint8_t day, uint8_t hour, uint8_t min, uint8_t sec) {
    Serial.print(year);
    Serial.print('/');
    print2d(month);
    Serial.print('/');
    print2d(day);
    Serial.print(' ');
    print2d(hour);
    Serial.print(':');
    print2d(min);
    Serial.print(':');
    print2d(sec);
    Serial.println();
}

void print2d(uint8_t val) {
    if (val < 10) {
        Serial.print("0");
    }
    Serial.print(val);
}


